#include <kernel.h>

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include <EGL/egl.h>
#include <EGL/eglext.h>

#include <JavaScriptCore.h>

#define SET_CONSTANT_PROPERTY(str) \
name = JSStringCreateWithUTF8CString(#str); \
JSObjectSetProperty(ctx, obj, name, JSValueMakeNumber(ctx, (double)GL_##str), kJSPropertyAttributeReadOnly, excp); \
JSStringRelease(name);

#define DELETE_CONSTANT_PROPERTY(str) \
name = JSStringCreateWithUTF8CString(#str); \
JSObjectDeleteProperty(ctx, obj, name, excp); \
JSStringRelease(name);

#define SET_CONSTANT_PROPERTY_NOPREFIX(str) \
name = JSStringCreateWithUTF8CString(#str); \
JSObjectSetProperty(ctx, obj, name, JSValueMakeNumber(ctx, (double)str), kJSPropertyAttributeReadOnly, excp); \
JSStringRelease(name);

void webglAddConstantProperties(JSContextRef ctx, JSObjectRef obj, JSValueRef *excp)
{
	JSStringRef name;

	/* ClearBufferMask */
	SET_CONSTANT_PROPERTY(DEPTH_BUFFER_BIT)
	SET_CONSTANT_PROPERTY(STENCIL_BUFFER_BIT)
	SET_CONSTANT_PROPERTY(COLOR_BUFFER_BIT)

		/* BeginMode */
	SET_CONSTANT_PROPERTY(POINTS)
	SET_CONSTANT_PROPERTY(LINES)
	SET_CONSTANT_PROPERTY(LINE_LOOP)
	SET_CONSTANT_PROPERTY(LINE_STRIP)
	SET_CONSTANT_PROPERTY(TRIANGLES)
	SET_CONSTANT_PROPERTY(TRIANGLE_STRIP)
	SET_CONSTANT_PROPERTY(TRIANGLE_FAN)

	/* BlendingFactorDest */
	SET_CONSTANT_PROPERTY(ZERO)
	SET_CONSTANT_PROPERTY(ONE)
	SET_CONSTANT_PROPERTY(SRC_COLOR)
	SET_CONSTANT_PROPERTY(ONE_MINUS_SRC_COLOR)
	SET_CONSTANT_PROPERTY(SRC_ALPHA)
	SET_CONSTANT_PROPERTY(ONE_MINUS_SRC_ALPHA)
	SET_CONSTANT_PROPERTY(DST_ALPHA)
	SET_CONSTANT_PROPERTY(ONE_MINUS_DST_ALPHA)

	/* BlendingFactorSrc */
	SET_CONSTANT_PROPERTY(DST_COLOR)
	SET_CONSTANT_PROPERTY(ONE_MINUS_DST_COLOR)
	SET_CONSTANT_PROPERTY(SRC_ALPHA_SATURATE)

	/* BlendEquationSeparate */
	SET_CONSTANT_PROPERTY(FUNC_ADD)
	SET_CONSTANT_PROPERTY(BLEND_EQUATION)
	SET_CONSTANT_PROPERTY(BLEND_EQUATION_RGB)
	SET_CONSTANT_PROPERTY(BLEND_EQUATION_ALPHA)

	/* BlendSubtract */
	SET_CONSTANT_PROPERTY(FUNC_SUBTRACT)
	SET_CONSTANT_PROPERTY(FUNC_REVERSE_SUBTRACT)

	/* Separate Blend Functions */
	SET_CONSTANT_PROPERTY(BLEND_DST_RGB)
	SET_CONSTANT_PROPERTY(BLEND_SRC_RGB)
	SET_CONSTANT_PROPERTY(BLEND_DST_ALPHA)
	SET_CONSTANT_PROPERTY(BLEND_SRC_ALPHA)
	SET_CONSTANT_PROPERTY(CONSTANT_COLOR)
	SET_CONSTANT_PROPERTY(ONE_MINUS_CONSTANT_COLOR)
	SET_CONSTANT_PROPERTY(CONSTANT_ALPHA)
	SET_CONSTANT_PROPERTY(ONE_MINUS_CONSTANT_ALPHA)
	SET_CONSTANT_PROPERTY(BLEND_COLOR)


	/* Buffer Objects */
	SET_CONSTANT_PROPERTY(ARRAY_BUFFER)
	SET_CONSTANT_PROPERTY(ELEMENT_ARRAY_BUFFER)
	SET_CONSTANT_PROPERTY(ARRAY_BUFFER_BINDING)
	SET_CONSTANT_PROPERTY(ELEMENT_ARRAY_BUFFER_BINDING)

	SET_CONSTANT_PROPERTY(STREAM_DRAW)
	SET_CONSTANT_PROPERTY(STATIC_DRAW)
	SET_CONSTANT_PROPERTY(DYNAMIC_DRAW)

	SET_CONSTANT_PROPERTY(BUFFER_SIZE)
	SET_CONSTANT_PROPERTY(BUFFER_USAGE)

	SET_CONSTANT_PROPERTY(CURRENT_VERTEX_ATTRIB)

	/* CullFaceMode */
	SET_CONSTANT_PROPERTY(FRONT)
	SET_CONSTANT_PROPERTY(BACK)
	SET_CONSTANT_PROPERTY(FRONT_AND_BACK)

	/* EnableCap */
	SET_CONSTANT_PROPERTY(TEXTURE_2D)
	SET_CONSTANT_PROPERTY(CULL_FACE)
	SET_CONSTANT_PROPERTY(BLEND)
	SET_CONSTANT_PROPERTY(DITHER)
	SET_CONSTANT_PROPERTY(STENCIL_TEST)
	SET_CONSTANT_PROPERTY(DEPTH_TEST)
	SET_CONSTANT_PROPERTY(SCISSOR_TEST)
	SET_CONSTANT_PROPERTY(POLYGON_OFFSET_FILL)
	SET_CONSTANT_PROPERTY(SAMPLE_ALPHA_TO_COVERAGE)
	SET_CONSTANT_PROPERTY(SAMPLE_COVERAGE)

	/* ErrorCode */
	SET_CONSTANT_PROPERTY(NO_ERROR)
	SET_CONSTANT_PROPERTY(INVALID_ENUM)
	SET_CONSTANT_PROPERTY(INVALID_VALUE)
	SET_CONSTANT_PROPERTY(INVALID_OPERATION)
	SET_CONSTANT_PROPERTY(OUT_OF_MEMORY)

	/* FrontFaceDirection */
	SET_CONSTANT_PROPERTY(CW)
	SET_CONSTANT_PROPERTY(CCW)

	/* GetPName */
	SET_CONSTANT_PROPERTY(LINE_WIDTH)
	SET_CONSTANT_PROPERTY(ALIASED_POINT_SIZE_RANGE)
	SET_CONSTANT_PROPERTY(ALIASED_LINE_WIDTH_RANGE)
	SET_CONSTANT_PROPERTY(CULL_FACE_MODE)
	SET_CONSTANT_PROPERTY(FRONT_FACE)
	SET_CONSTANT_PROPERTY(DEPTH_RANGE)
	SET_CONSTANT_PROPERTY(DEPTH_WRITEMASK)
	SET_CONSTANT_PROPERTY(DEPTH_CLEAR_VALUE)
	SET_CONSTANT_PROPERTY(DEPTH_FUNC)
	SET_CONSTANT_PROPERTY(STENCIL_CLEAR_VALUE)
	SET_CONSTANT_PROPERTY(STENCIL_FUNC)
	SET_CONSTANT_PROPERTY(STENCIL_FAIL)
	SET_CONSTANT_PROPERTY(STENCIL_PASS_DEPTH_FAIL)
	SET_CONSTANT_PROPERTY(STENCIL_PASS_DEPTH_PASS)
	SET_CONSTANT_PROPERTY(STENCIL_REF)
	SET_CONSTANT_PROPERTY(STENCIL_VALUE_MASK)
	SET_CONSTANT_PROPERTY(STENCIL_WRITEMASK)
	SET_CONSTANT_PROPERTY(STENCIL_BACK_FUNC)
	SET_CONSTANT_PROPERTY(STENCIL_BACK_FAIL)
	SET_CONSTANT_PROPERTY(STENCIL_BACK_PASS_DEPTH_FAIL)
	SET_CONSTANT_PROPERTY(STENCIL_BACK_PASS_DEPTH_PASS)
	SET_CONSTANT_PROPERTY(STENCIL_BACK_REF)
	SET_CONSTANT_PROPERTY(STENCIL_BACK_VALUE_MASK)
	SET_CONSTANT_PROPERTY(STENCIL_BACK_WRITEMASK)
	SET_CONSTANT_PROPERTY(VIEWPORT)
	SET_CONSTANT_PROPERTY(SCISSOR_BOX)
	SET_CONSTANT_PROPERTY(COLOR_CLEAR_VALUE)
	SET_CONSTANT_PROPERTY(COLOR_WRITEMASK)
	SET_CONSTANT_PROPERTY(UNPACK_ALIGNMENT)
	SET_CONSTANT_PROPERTY(PACK_ALIGNMENT)
	SET_CONSTANT_PROPERTY(MAX_TEXTURE_SIZE)
	SET_CONSTANT_PROPERTY(MAX_VIEWPORT_DIMS)
	SET_CONSTANT_PROPERTY(SUBPIXEL_BITS)
	SET_CONSTANT_PROPERTY(RED_BITS)
	SET_CONSTANT_PROPERTY(GREEN_BITS)
	SET_CONSTANT_PROPERTY(BLUE_BITS)
	SET_CONSTANT_PROPERTY(ALPHA_BITS)
	SET_CONSTANT_PROPERTY(DEPTH_BITS)
	SET_CONSTANT_PROPERTY(STENCIL_BITS)
	SET_CONSTANT_PROPERTY(POLYGON_OFFSET_UNITS)
	SET_CONSTANT_PROPERTY(POLYGON_OFFSET_FACTOR)
	SET_CONSTANT_PROPERTY(TEXTURE_BINDING_2D)
	SET_CONSTANT_PROPERTY(SAMPLE_BUFFERS)
	SET_CONSTANT_PROPERTY(SAMPLES)
	SET_CONSTANT_PROPERTY(SAMPLE_COVERAGE_VALUE)
	SET_CONSTANT_PROPERTY(SAMPLE_COVERAGE_INVERT)

	SET_CONSTANT_PROPERTY(NUM_COMPRESSED_TEXTURE_FORMATS)
	SET_CONSTANT_PROPERTY(COMPRESSED_TEXTURE_FORMATS)

	/* HintMode */
	SET_CONSTANT_PROPERTY(DONT_CARE)
	SET_CONSTANT_PROPERTY(FASTEST)
	SET_CONSTANT_PROPERTY(NICEST)

	/* HintTarget */
	SET_CONSTANT_PROPERTY(GENERATE_MIPMAP_HINT)

	/* DataType */
	SET_CONSTANT_PROPERTY(BYTE)
	SET_CONSTANT_PROPERTY(UNSIGNED_BYTE)
	SET_CONSTANT_PROPERTY(SHORT)
	SET_CONSTANT_PROPERTY(UNSIGNED_SHORT)
	SET_CONSTANT_PROPERTY(INT)
	SET_CONSTANT_PROPERTY(UNSIGNED_INT)
	SET_CONSTANT_PROPERTY(FLOAT)
	SET_CONSTANT_PROPERTY(FIXED)

	/* PixelFormat */
	SET_CONSTANT_PROPERTY(DEPTH_COMPONENT)
	SET_CONSTANT_PROPERTY(ALPHA)
	SET_CONSTANT_PROPERTY(RGB)
	SET_CONSTANT_PROPERTY(RGBA)
	SET_CONSTANT_PROPERTY(LUMINANCE)
	SET_CONSTANT_PROPERTY(LUMINANCE_ALPHA)

	/* PixelType */
	SET_CONSTANT_PROPERTY(UNSIGNED_SHORT_4_4_4_4)
	SET_CONSTANT_PROPERTY(UNSIGNED_SHORT_5_5_5_1)
	SET_CONSTANT_PROPERTY(UNSIGNED_SHORT_5_6_5)

	/* Shaders */
	SET_CONSTANT_PROPERTY(FRAGMENT_SHADER)
	SET_CONSTANT_PROPERTY(VERTEX_SHADER)
	SET_CONSTANT_PROPERTY(MAX_VERTEX_ATTRIBS)
	SET_CONSTANT_PROPERTY(MAX_VERTEX_UNIFORM_VECTORS)
	SET_CONSTANT_PROPERTY(MAX_VARYING_VECTORS)
	SET_CONSTANT_PROPERTY(MAX_COMBINED_TEXTURE_IMAGE_UNITS)
	SET_CONSTANT_PROPERTY(MAX_VERTEX_TEXTURE_IMAGE_UNITS)
	SET_CONSTANT_PROPERTY(MAX_TEXTURE_IMAGE_UNITS)
	SET_CONSTANT_PROPERTY(MAX_FRAGMENT_UNIFORM_VECTORS)
	SET_CONSTANT_PROPERTY(SHADER_TYPE)
	SET_CONSTANT_PROPERTY(DELETE_STATUS)
	SET_CONSTANT_PROPERTY(LINK_STATUS)
	SET_CONSTANT_PROPERTY(VALIDATE_STATUS)
	SET_CONSTANT_PROPERTY(ATTACHED_SHADERS)
	SET_CONSTANT_PROPERTY(ACTIVE_UNIFORMS)
	SET_CONSTANT_PROPERTY(ACTIVE_UNIFORM_MAX_LENGTH)
	SET_CONSTANT_PROPERTY(ACTIVE_ATTRIBUTES)
	SET_CONSTANT_PROPERTY(ACTIVE_ATTRIBUTE_MAX_LENGTH)
	SET_CONSTANT_PROPERTY(SHADING_LANGUAGE_VERSION)
	SET_CONSTANT_PROPERTY(CURRENT_PROGRAM)

	/* StencilFunction */
	SET_CONSTANT_PROPERTY(NEVER)
	SET_CONSTANT_PROPERTY(LESS)
	SET_CONSTANT_PROPERTY(EQUAL)
	SET_CONSTANT_PROPERTY(LEQUAL)
	SET_CONSTANT_PROPERTY(GREATER)
	SET_CONSTANT_PROPERTY(NOTEQUAL)
	SET_CONSTANT_PROPERTY(GEQUAL)
	SET_CONSTANT_PROPERTY(ALWAYS)

	/* StencilOp */
	SET_CONSTANT_PROPERTY(KEEP)
	SET_CONSTANT_PROPERTY(REPLACE)
	SET_CONSTANT_PROPERTY(INCR)
	SET_CONSTANT_PROPERTY(DECR)
	SET_CONSTANT_PROPERTY(INVERT)
	SET_CONSTANT_PROPERTY(INCR_WRAP)
	SET_CONSTANT_PROPERTY(DECR_WRAP)

	/* StringName */
	SET_CONSTANT_PROPERTY(VENDOR)
	SET_CONSTANT_PROPERTY(RENDERER)
	SET_CONSTANT_PROPERTY(VERSION)
	SET_CONSTANT_PROPERTY(EXTENSIONS)

	/* TextureMagFilter */
	SET_CONSTANT_PROPERTY(NEAREST)
	SET_CONSTANT_PROPERTY(LINEAR)

	/* TextureMinFilter */
	SET_CONSTANT_PROPERTY(NEAREST_MIPMAP_NEAREST)
	SET_CONSTANT_PROPERTY(LINEAR_MIPMAP_NEAREST)
	SET_CONSTANT_PROPERTY(NEAREST_MIPMAP_LINEAR)
	SET_CONSTANT_PROPERTY(LINEAR_MIPMAP_LINEAR)

	/* TextureParameterName */
	SET_CONSTANT_PROPERTY(TEXTURE_MAG_FILTER)
	SET_CONSTANT_PROPERTY(TEXTURE_MIN_FILTER)
	SET_CONSTANT_PROPERTY(TEXTURE_WRAP_S)
	SET_CONSTANT_PROPERTY(TEXTURE_WRAP_T)

	/* TextureTarget */
	SET_CONSTANT_PROPERTY(TEXTURE)

	SET_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP)
	SET_CONSTANT_PROPERTY(TEXTURE_BINDING_CUBE_MAP)
	SET_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_POSITIVE_X)
	SET_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_NEGATIVE_X)
	SET_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_POSITIVE_Y)
	SET_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_NEGATIVE_Y)
	SET_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_POSITIVE_Z)
	SET_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_NEGATIVE_Z)
	SET_CONSTANT_PROPERTY(MAX_CUBE_MAP_TEXTURE_SIZE)

	/* TextureUnit */
	SET_CONSTANT_PROPERTY(TEXTURE0)
	SET_CONSTANT_PROPERTY(TEXTURE1)
	SET_CONSTANT_PROPERTY(TEXTURE2)
	SET_CONSTANT_PROPERTY(TEXTURE3)
	SET_CONSTANT_PROPERTY(TEXTURE4)
	SET_CONSTANT_PROPERTY(TEXTURE5)
	SET_CONSTANT_PROPERTY(TEXTURE6)
	SET_CONSTANT_PROPERTY(TEXTURE7)
	SET_CONSTANT_PROPERTY(TEXTURE8)
	SET_CONSTANT_PROPERTY(TEXTURE9)
	SET_CONSTANT_PROPERTY(TEXTURE10)
	SET_CONSTANT_PROPERTY(TEXTURE11)
	SET_CONSTANT_PROPERTY(TEXTURE12)
	SET_CONSTANT_PROPERTY(TEXTURE13)
	SET_CONSTANT_PROPERTY(TEXTURE14)
	SET_CONSTANT_PROPERTY(TEXTURE15)
	SET_CONSTANT_PROPERTY(TEXTURE16)
	SET_CONSTANT_PROPERTY(TEXTURE17)
	SET_CONSTANT_PROPERTY(TEXTURE18)
	SET_CONSTANT_PROPERTY(TEXTURE19)
	SET_CONSTANT_PROPERTY(TEXTURE20)
	SET_CONSTANT_PROPERTY(TEXTURE21)
	SET_CONSTANT_PROPERTY(TEXTURE22)
	SET_CONSTANT_PROPERTY(TEXTURE23)
	SET_CONSTANT_PROPERTY(TEXTURE24)
	SET_CONSTANT_PROPERTY(TEXTURE25)
	SET_CONSTANT_PROPERTY(TEXTURE26)
	SET_CONSTANT_PROPERTY(TEXTURE27)
	SET_CONSTANT_PROPERTY(TEXTURE28)
	SET_CONSTANT_PROPERTY(TEXTURE29)
	SET_CONSTANT_PROPERTY(TEXTURE30)
	SET_CONSTANT_PROPERTY(TEXTURE31)
	SET_CONSTANT_PROPERTY(ACTIVE_TEXTURE)

	/* TextureWrapMode */
	SET_CONSTANT_PROPERTY(REPEAT)
	SET_CONSTANT_PROPERTY(CLAMP_TO_EDGE)
	SET_CONSTANT_PROPERTY(MIRRORED_REPEAT)

	/* Uniform Types */
	SET_CONSTANT_PROPERTY(FLOAT_VEC2)
	SET_CONSTANT_PROPERTY(FLOAT_VEC3)
	SET_CONSTANT_PROPERTY(FLOAT_VEC4)
	SET_CONSTANT_PROPERTY(INT_VEC2)
	SET_CONSTANT_PROPERTY(INT_VEC3)
	SET_CONSTANT_PROPERTY(INT_VEC4)
	SET_CONSTANT_PROPERTY(BOOL)
	SET_CONSTANT_PROPERTY(BOOL_VEC2)
	SET_CONSTANT_PROPERTY(BOOL_VEC3)
	SET_CONSTANT_PROPERTY(BOOL_VEC4)
	SET_CONSTANT_PROPERTY(FLOAT_MAT2)
	SET_CONSTANT_PROPERTY(FLOAT_MAT3)
	SET_CONSTANT_PROPERTY(FLOAT_MAT4)
	SET_CONSTANT_PROPERTY(SAMPLER_2D)
	SET_CONSTANT_PROPERTY(SAMPLER_CUBE)

	/* Vertex Arrays */
	SET_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_ENABLED)
	SET_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_SIZE)
	SET_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_STRIDE)
	SET_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_TYPE)
	SET_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_NORMALIZED)
	SET_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_POINTER)
	SET_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_BUFFER_BINDING)

	/* Read Format */
	SET_CONSTANT_PROPERTY(IMPLEMENTATION_COLOR_READ_TYPE)
	SET_CONSTANT_PROPERTY(IMPLEMENTATION_COLOR_READ_FORMAT)

	/* Shader Source */
	SET_CONSTANT_PROPERTY(COMPILE_STATUS)
	SET_CONSTANT_PROPERTY(INFO_LOG_LENGTH)
	SET_CONSTANT_PROPERTY(SHADER_SOURCE_LENGTH)
	SET_CONSTANT_PROPERTY(SHADER_COMPILER)

	/* Shader Binary */
	SET_CONSTANT_PROPERTY(SHADER_BINARY_FORMATS)
	SET_CONSTANT_PROPERTY(NUM_SHADER_BINARY_FORMATS)

	/* Shader Precision-Specified Types */
	SET_CONSTANT_PROPERTY(LOW_FLOAT)
	SET_CONSTANT_PROPERTY(MEDIUM_FLOAT)
	SET_CONSTANT_PROPERTY(HIGH_FLOAT)
	SET_CONSTANT_PROPERTY(LOW_INT)
	SET_CONSTANT_PROPERTY(MEDIUM_INT)
	SET_CONSTANT_PROPERTY(HIGH_INT)

	/* Framebuffer Object. */
	SET_CONSTANT_PROPERTY(FRAMEBUFFER)
	SET_CONSTANT_PROPERTY(RENDERBUFFER)

	SET_CONSTANT_PROPERTY(RGBA4)
	SET_CONSTANT_PROPERTY(RGB5_A1)
	SET_CONSTANT_PROPERTY(RGB565)
	SET_CONSTANT_PROPERTY(DEPTH_COMPONENT16)
	SET_CONSTANT_PROPERTY(STENCIL_INDEX)
	SET_CONSTANT_PROPERTY(STENCIL_INDEX8)

	SET_CONSTANT_PROPERTY(RENDERBUFFER_WIDTH)
	SET_CONSTANT_PROPERTY(RENDERBUFFER_HEIGHT)
	SET_CONSTANT_PROPERTY(RENDERBUFFER_INTERNAL_FORMAT)
	SET_CONSTANT_PROPERTY(RENDERBUFFER_RED_SIZE)
	SET_CONSTANT_PROPERTY(RENDERBUFFER_GREEN_SIZE)
	SET_CONSTANT_PROPERTY(RENDERBUFFER_BLUE_SIZE)
	SET_CONSTANT_PROPERTY(RENDERBUFFER_ALPHA_SIZE)
	SET_CONSTANT_PROPERTY(RENDERBUFFER_DEPTH_SIZE)
	SET_CONSTANT_PROPERTY(RENDERBUFFER_STENCIL_SIZE)

	SET_CONSTANT_PROPERTY(FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE)
	SET_CONSTANT_PROPERTY(FRAMEBUFFER_ATTACHMENT_OBJECT_NAME)
	SET_CONSTANT_PROPERTY(FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL)
	SET_CONSTANT_PROPERTY(FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE)

	SET_CONSTANT_PROPERTY(COLOR_ATTACHMENT0)
	SET_CONSTANT_PROPERTY(DEPTH_ATTACHMENT)
	SET_CONSTANT_PROPERTY(STENCIL_ATTACHMENT)

	SET_CONSTANT_PROPERTY(NONE)

	SET_CONSTANT_PROPERTY(FRAMEBUFFER_COMPLETE)
	SET_CONSTANT_PROPERTY(FRAMEBUFFER_INCOMPLETE_ATTACHMENT)
	SET_CONSTANT_PROPERTY(FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT)
	SET_CONSTANT_PROPERTY(FRAMEBUFFER_INCOMPLETE_DIMENSIONS)
	SET_CONSTANT_PROPERTY(FRAMEBUFFER_UNSUPPORTED)

	SET_CONSTANT_PROPERTY(FRAMEBUFFER_BINDING)
	SET_CONSTANT_PROPERTY(RENDERBUFFER_BINDING)
	SET_CONSTANT_PROPERTY(MAX_RENDERBUFFER_SIZE)

	SET_CONSTANT_PROPERTY(INVALID_FRAMEBUFFER_OPERATION)

	SET_CONSTANT_PROPERTY_NOPREFIX(PSP2_WINDOW_960X544)
	SET_CONSTANT_PROPERTY_NOPREFIX(PSP2_WINDOW_480X272)
	SET_CONSTANT_PROPERTY_NOPREFIX(PSP2_WINDOW_640X368)
	SET_CONSTANT_PROPERTY_NOPREFIX(PSP2_WINDOW_720X408)
	SET_CONSTANT_PROPERTY_NOPREFIX(PSP2_WINDOW_1280X725)
	SET_CONSTANT_PROPERTY_NOPREFIX(PSP2_WINDOW_1920X1088)

	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_DONT_CARE)

	/* Config attributes */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_BUFFER_SIZE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_ALPHA_SIZE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_BLUE_SIZE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_GREEN_SIZE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_RED_SIZE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_DEPTH_SIZE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_STENCIL_SIZE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_CONFIG_CAVEAT)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_CONFIG_ID)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_LEVEL)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_MAX_PBUFFER_HEIGHT)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_MAX_PBUFFER_PIXELS)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_MAX_PBUFFER_WIDTH)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_NATIVE_RENDERABLE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_NATIVE_VISUAL_ID)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_NATIVE_VISUAL_TYPE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_SAMPLES)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_SAMPLE_BUFFERS)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_SURFACE_TYPE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_TRANSPARENT_TYPE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_TRANSPARENT_BLUE_VALUE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_TRANSPARENT_GREEN_VALUE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_TRANSPARENT_RED_VALUE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_NONE)	/* Attrib list terminator */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_BIND_TO_TEXTURE_RGB)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_BIND_TO_TEXTURE_RGBA)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_MIN_SWAP_INTERVAL)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_MAX_SWAP_INTERVAL)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_LUMINANCE_SIZE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_ALPHA_MASK_SIZE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_COLOR_BUFFER_TYPE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_RENDERABLE_TYPE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_MATCH_NATIVE_PIXMAP)	/* Pseudo-attribute (not queryable) */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_CONFORMANT)

	/* Reserved 0x3041-0x304F for additional config attributes */

	/* Config attribute values */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_SLOW_CONFIG)	/* EGL_CONFIG_CAVEAT value */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_NON_CONFORMANT_CONFIG)	/* EGL_CONFIG_CAVEAT value */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_TRANSPARENT_RGB)	/* EGL_TRANSPARENT_TYPE value */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_RGB_BUFFER)	/* EGL_COLOR_BUFFER_TYPE value */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_LUMINANCE_BUFFER)	/* EGL_COLOR_BUFFER_TYPE value */

	/* More config attribute values, for EGL_TEXTURE_FORMAT */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_NO_TEXTURE)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_TEXTURE_RGB)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_TEXTURE_RGBA)
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_TEXTURE_2D)

	/* Config attribute mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_PBUFFER_BIT)	/* EGL_SURFACE_TYPE mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_PIXMAP_BIT)	/* EGL_SURFACE_TYPE mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_WINDOW_BIT)	/* EGL_SURFACE_TYPE mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_VG_COLORSPACE_LINEAR_BIT)	/* EGL_SURFACE_TYPE mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_VG_ALPHA_FORMAT_PRE_BIT)	/* EGL_SURFACE_TYPE mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_MULTISAMPLE_RESOLVE_BOX_BIT)	/* EGL_SURFACE_TYPE mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_SWAP_BEHAVIOR_PRESERVED_BIT)	/* EGL_SURFACE_TYPE mask bits */

	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_OPENGL_ES_BIT)	/* EGL_RENDERABLE_TYPE mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_OPENVG_BIT)	/* EGL_RENDERABLE_TYPE mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_OPENGL_ES2_BIT)	/* EGL_RENDERABLE_TYPE mask bits */
	SET_CONSTANT_PROPERTY_NOPREFIX(EGL_OPENGL_BIT)	/* EGL_RENDERABLE_TYPE mask bits */
}

void webglRemoveConstantProperties(JSContextRef ctx, JSObjectRef obj, JSValueRef *excp)
{
	JSStringRef name;

	/* ClearBufferMask */
	DELETE_CONSTANT_PROPERTY(DEPTH_BUFFER_BIT)
	DELETE_CONSTANT_PROPERTY(STENCIL_BUFFER_BIT)
	DELETE_CONSTANT_PROPERTY(COLOR_BUFFER_BIT)

	/* BeginMode */
	DELETE_CONSTANT_PROPERTY(POINTS)
	DELETE_CONSTANT_PROPERTY(LINES)
	DELETE_CONSTANT_PROPERTY(LINE_LOOP)
	DELETE_CONSTANT_PROPERTY(LINE_STRIP)
	DELETE_CONSTANT_PROPERTY(TRIANGLES)
	DELETE_CONSTANT_PROPERTY(TRIANGLE_STRIP)
	DELETE_CONSTANT_PROPERTY(TRIANGLE_FAN)

	/* BlendingFactorDest */
	DELETE_CONSTANT_PROPERTY(ZERO)
	DELETE_CONSTANT_PROPERTY(ONE)
	DELETE_CONSTANT_PROPERTY(SRC_COLOR)
	DELETE_CONSTANT_PROPERTY(ONE_MINUS_SRC_COLOR)
	DELETE_CONSTANT_PROPERTY(SRC_ALPHA)
	DELETE_CONSTANT_PROPERTY(ONE_MINUS_SRC_ALPHA)
	DELETE_CONSTANT_PROPERTY(DST_ALPHA)
	DELETE_CONSTANT_PROPERTY(ONE_MINUS_DST_ALPHA)

	/* BlendingFactorSrc */
	DELETE_CONSTANT_PROPERTY(DST_COLOR)
	DELETE_CONSTANT_PROPERTY(ONE_MINUS_DST_COLOR)
	DELETE_CONSTANT_PROPERTY(SRC_ALPHA_SATURATE)

	/* BlendEquationSeparate */
	DELETE_CONSTANT_PROPERTY(FUNC_ADD)
	DELETE_CONSTANT_PROPERTY(BLEND_EQUATION)
	DELETE_CONSTANT_PROPERTY(BLEND_EQUATION_RGB)
	DELETE_CONSTANT_PROPERTY(BLEND_EQUATION_ALPHA)

	/* BlendSubtract */
	DELETE_CONSTANT_PROPERTY(FUNC_SUBTRACT)
	DELETE_CONSTANT_PROPERTY(FUNC_REVERSE_SUBTRACT)

	/* Separate Blend Functions */
	DELETE_CONSTANT_PROPERTY(BLEND_DST_RGB)
	DELETE_CONSTANT_PROPERTY(BLEND_SRC_RGB)
	DELETE_CONSTANT_PROPERTY(BLEND_DST_ALPHA)
	DELETE_CONSTANT_PROPERTY(BLEND_SRC_ALPHA)
	DELETE_CONSTANT_PROPERTY(CONSTANT_COLOR)
	DELETE_CONSTANT_PROPERTY(ONE_MINUS_CONSTANT_COLOR)
	DELETE_CONSTANT_PROPERTY(CONSTANT_ALPHA)
	DELETE_CONSTANT_PROPERTY(ONE_MINUS_CONSTANT_ALPHA)
	DELETE_CONSTANT_PROPERTY(BLEND_COLOR)


	/* Buffer Objects */
	DELETE_CONSTANT_PROPERTY(ARRAY_BUFFER)
	DELETE_CONSTANT_PROPERTY(ELEMENT_ARRAY_BUFFER)
	DELETE_CONSTANT_PROPERTY(ARRAY_BUFFER_BINDING)
	DELETE_CONSTANT_PROPERTY(ELEMENT_ARRAY_BUFFER_BINDING)

	DELETE_CONSTANT_PROPERTY(STREAM_DRAW)
	DELETE_CONSTANT_PROPERTY(STATIC_DRAW)
	DELETE_CONSTANT_PROPERTY(DYNAMIC_DRAW)

	DELETE_CONSTANT_PROPERTY(BUFFER_SIZE)
	DELETE_CONSTANT_PROPERTY(BUFFER_USAGE)

	DELETE_CONSTANT_PROPERTY(CURRENT_VERTEX_ATTRIB)

	/* CullFaceMode */
	DELETE_CONSTANT_PROPERTY(FRONT)
	DELETE_CONSTANT_PROPERTY(BACK)
	DELETE_CONSTANT_PROPERTY(FRONT_AND_BACK)

	/* EnableCap */
	DELETE_CONSTANT_PROPERTY(TEXTURE_2D)
	DELETE_CONSTANT_PROPERTY(CULL_FACE)
	DELETE_CONSTANT_PROPERTY(BLEND)
	DELETE_CONSTANT_PROPERTY(DITHER)
	DELETE_CONSTANT_PROPERTY(STENCIL_TEST)
	DELETE_CONSTANT_PROPERTY(DEPTH_TEST)
	DELETE_CONSTANT_PROPERTY(SCISSOR_TEST)
	DELETE_CONSTANT_PROPERTY(POLYGON_OFFSET_FILL)
	DELETE_CONSTANT_PROPERTY(SAMPLE_ALPHA_TO_COVERAGE)
	DELETE_CONSTANT_PROPERTY(SAMPLE_COVERAGE)

	/* ErrorCode */
	DELETE_CONSTANT_PROPERTY(NO_ERROR)
	DELETE_CONSTANT_PROPERTY(INVALID_ENUM)
	DELETE_CONSTANT_PROPERTY(INVALID_VALUE)
	DELETE_CONSTANT_PROPERTY(INVALID_OPERATION)
	DELETE_CONSTANT_PROPERTY(OUT_OF_MEMORY)

	/* FrontFaceDirection */
	DELETE_CONSTANT_PROPERTY(CW)
	DELETE_CONSTANT_PROPERTY(CCW)

	/* GetPName */
	DELETE_CONSTANT_PROPERTY(LINE_WIDTH)
	DELETE_CONSTANT_PROPERTY(ALIASED_POINT_SIZE_RANGE)
	DELETE_CONSTANT_PROPERTY(ALIASED_LINE_WIDTH_RANGE)
	DELETE_CONSTANT_PROPERTY(CULL_FACE_MODE)
	DELETE_CONSTANT_PROPERTY(FRONT_FACE)
	DELETE_CONSTANT_PROPERTY(DEPTH_RANGE)
	DELETE_CONSTANT_PROPERTY(DEPTH_WRITEMASK)
	DELETE_CONSTANT_PROPERTY(DEPTH_CLEAR_VALUE)
	DELETE_CONSTANT_PROPERTY(DEPTH_FUNC)
	DELETE_CONSTANT_PROPERTY(STENCIL_CLEAR_VALUE)
	DELETE_CONSTANT_PROPERTY(STENCIL_FUNC)
	DELETE_CONSTANT_PROPERTY(STENCIL_FAIL)
	DELETE_CONSTANT_PROPERTY(STENCIL_PASS_DEPTH_FAIL)
	DELETE_CONSTANT_PROPERTY(STENCIL_PASS_DEPTH_PASS)
	DELETE_CONSTANT_PROPERTY(STENCIL_REF)
	DELETE_CONSTANT_PROPERTY(STENCIL_VALUE_MASK)
	DELETE_CONSTANT_PROPERTY(STENCIL_WRITEMASK)
	DELETE_CONSTANT_PROPERTY(STENCIL_BACK_FUNC)
	DELETE_CONSTANT_PROPERTY(STENCIL_BACK_FAIL)
	DELETE_CONSTANT_PROPERTY(STENCIL_BACK_PASS_DEPTH_FAIL)
	DELETE_CONSTANT_PROPERTY(STENCIL_BACK_PASS_DEPTH_PASS)
	DELETE_CONSTANT_PROPERTY(STENCIL_BACK_REF)
	DELETE_CONSTANT_PROPERTY(STENCIL_BACK_VALUE_MASK)
	DELETE_CONSTANT_PROPERTY(STENCIL_BACK_WRITEMASK)
	DELETE_CONSTANT_PROPERTY(VIEWPORT)
	DELETE_CONSTANT_PROPERTY(SCISSOR_BOX)
	DELETE_CONSTANT_PROPERTY(COLOR_CLEAR_VALUE)
	DELETE_CONSTANT_PROPERTY(COLOR_WRITEMASK)
	DELETE_CONSTANT_PROPERTY(UNPACK_ALIGNMENT)
	DELETE_CONSTANT_PROPERTY(PACK_ALIGNMENT)
	DELETE_CONSTANT_PROPERTY(MAX_TEXTURE_SIZE)
	DELETE_CONSTANT_PROPERTY(MAX_VIEWPORT_DIMS)
	DELETE_CONSTANT_PROPERTY(SUBPIXEL_BITS)
	DELETE_CONSTANT_PROPERTY(RED_BITS)
	DELETE_CONSTANT_PROPERTY(GREEN_BITS)
	DELETE_CONSTANT_PROPERTY(BLUE_BITS)
	DELETE_CONSTANT_PROPERTY(ALPHA_BITS)
	DELETE_CONSTANT_PROPERTY(DEPTH_BITS)
	DELETE_CONSTANT_PROPERTY(STENCIL_BITS)
	DELETE_CONSTANT_PROPERTY(POLYGON_OFFSET_UNITS)
	DELETE_CONSTANT_PROPERTY(POLYGON_OFFSET_FACTOR)
	DELETE_CONSTANT_PROPERTY(TEXTURE_BINDING_2D)
	DELETE_CONSTANT_PROPERTY(SAMPLE_BUFFERS)
	DELETE_CONSTANT_PROPERTY(SAMPLES)
	DELETE_CONSTANT_PROPERTY(SAMPLE_COVERAGE_VALUE)
	DELETE_CONSTANT_PROPERTY(SAMPLE_COVERAGE_INVERT)

	DELETE_CONSTANT_PROPERTY(NUM_COMPRESSED_TEXTURE_FORMATS)
	DELETE_CONSTANT_PROPERTY(COMPRESSED_TEXTURE_FORMATS)

	/* HintMode */
	DELETE_CONSTANT_PROPERTY(DONT_CARE)
	DELETE_CONSTANT_PROPERTY(FASTEST)
	DELETE_CONSTANT_PROPERTY(NICEST)

	/* HintTarget */
	DELETE_CONSTANT_PROPERTY(GENERATE_MIPMAP_HINT)

	/* DataType */
	DELETE_CONSTANT_PROPERTY(BYTE)
	DELETE_CONSTANT_PROPERTY(UNSIGNED_BYTE)
	DELETE_CONSTANT_PROPERTY(SHORT)
	DELETE_CONSTANT_PROPERTY(UNSIGNED_SHORT)
	DELETE_CONSTANT_PROPERTY(INT)
	DELETE_CONSTANT_PROPERTY(UNSIGNED_INT)
	DELETE_CONSTANT_PROPERTY(FLOAT)
	DELETE_CONSTANT_PROPERTY(FIXED)

	/* PixelFormat */
	DELETE_CONSTANT_PROPERTY(DEPTH_COMPONENT)
	DELETE_CONSTANT_PROPERTY(ALPHA)
	DELETE_CONSTANT_PROPERTY(RGB)
	DELETE_CONSTANT_PROPERTY(RGBA)
	DELETE_CONSTANT_PROPERTY(LUMINANCE)
	DELETE_CONSTANT_PROPERTY(LUMINANCE_ALPHA)

	/* PixelType */
	DELETE_CONSTANT_PROPERTY(UNSIGNED_SHORT_4_4_4_4)
	DELETE_CONSTANT_PROPERTY(UNSIGNED_SHORT_5_5_5_1)
	DELETE_CONSTANT_PROPERTY(UNSIGNED_SHORT_5_6_5)

	/* Shaders */
	DELETE_CONSTANT_PROPERTY(FRAGMENT_SHADER)
	DELETE_CONSTANT_PROPERTY(VERTEX_SHADER)
	DELETE_CONSTANT_PROPERTY(MAX_VERTEX_ATTRIBS)
	DELETE_CONSTANT_PROPERTY(MAX_VERTEX_UNIFORM_VECTORS)
	DELETE_CONSTANT_PROPERTY(MAX_VARYING_VECTORS)
	DELETE_CONSTANT_PROPERTY(MAX_COMBINED_TEXTURE_IMAGE_UNITS)
	DELETE_CONSTANT_PROPERTY(MAX_VERTEX_TEXTURE_IMAGE_UNITS)
	DELETE_CONSTANT_PROPERTY(MAX_TEXTURE_IMAGE_UNITS)
	DELETE_CONSTANT_PROPERTY(MAX_FRAGMENT_UNIFORM_VECTORS)
	DELETE_CONSTANT_PROPERTY(SHADER_TYPE)
	DELETE_CONSTANT_PROPERTY(DELETE_STATUS)
	DELETE_CONSTANT_PROPERTY(LINK_STATUS)
	DELETE_CONSTANT_PROPERTY(VALIDATE_STATUS)
	DELETE_CONSTANT_PROPERTY(ATTACHED_SHADERS)
	DELETE_CONSTANT_PROPERTY(ACTIVE_UNIFORMS)
	DELETE_CONSTANT_PROPERTY(ACTIVE_UNIFORM_MAX_LENGTH)
	DELETE_CONSTANT_PROPERTY(ACTIVE_ATTRIBUTES)
	DELETE_CONSTANT_PROPERTY(ACTIVE_ATTRIBUTE_MAX_LENGTH)
	DELETE_CONSTANT_PROPERTY(SHADING_LANGUAGE_VERSION)
	DELETE_CONSTANT_PROPERTY(CURRENT_PROGRAM)

	/* StencilFunction */
	DELETE_CONSTANT_PROPERTY(NEVER)
	DELETE_CONSTANT_PROPERTY(LESS)
	DELETE_CONSTANT_PROPERTY(EQUAL)
	DELETE_CONSTANT_PROPERTY(LEQUAL)
	DELETE_CONSTANT_PROPERTY(GREATER)
	DELETE_CONSTANT_PROPERTY(NOTEQUAL)
	DELETE_CONSTANT_PROPERTY(GEQUAL)
	DELETE_CONSTANT_PROPERTY(ALWAYS)

	/* StencilOp */
	DELETE_CONSTANT_PROPERTY(KEEP)
	DELETE_CONSTANT_PROPERTY(REPLACE)
	DELETE_CONSTANT_PROPERTY(INCR)
	DELETE_CONSTANT_PROPERTY(DECR)
	DELETE_CONSTANT_PROPERTY(INVERT)
	DELETE_CONSTANT_PROPERTY(INCR_WRAP)
	DELETE_CONSTANT_PROPERTY(DECR_WRAP)

	/* StringName */
	DELETE_CONSTANT_PROPERTY(VENDOR)
	DELETE_CONSTANT_PROPERTY(RENDERER)
	DELETE_CONSTANT_PROPERTY(VERSION)
	DELETE_CONSTANT_PROPERTY(EXTENSIONS)

	/* TextureMagFilter */
	DELETE_CONSTANT_PROPERTY(NEAREST)
	DELETE_CONSTANT_PROPERTY(LINEAR)

	/* TextureMinFilter */
	DELETE_CONSTANT_PROPERTY(NEAREST_MIPMAP_NEAREST)
	DELETE_CONSTANT_PROPERTY(LINEAR_MIPMAP_NEAREST)
	DELETE_CONSTANT_PROPERTY(NEAREST_MIPMAP_LINEAR)
	DELETE_CONSTANT_PROPERTY(LINEAR_MIPMAP_LINEAR)

	/* TextureParameterName */
	DELETE_CONSTANT_PROPERTY(TEXTURE_MAG_FILTER)
	DELETE_CONSTANT_PROPERTY(TEXTURE_MIN_FILTER)
	DELETE_CONSTANT_PROPERTY(TEXTURE_WRAP_S)
	DELETE_CONSTANT_PROPERTY(TEXTURE_WRAP_T)

	/* TextureTarget */
	DELETE_CONSTANT_PROPERTY(TEXTURE)

	DELETE_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP)
	DELETE_CONSTANT_PROPERTY(TEXTURE_BINDING_CUBE_MAP)
	DELETE_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_POSITIVE_X)
	DELETE_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_NEGATIVE_X)
	DELETE_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_POSITIVE_Y)
	DELETE_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_NEGATIVE_Y)
	DELETE_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_POSITIVE_Z)
	DELETE_CONSTANT_PROPERTY(TEXTURE_CUBE_MAP_NEGATIVE_Z)
	DELETE_CONSTANT_PROPERTY(MAX_CUBE_MAP_TEXTURE_SIZE)

	/* TextureUnit */
	DELETE_CONSTANT_PROPERTY(TEXTURE0)
	DELETE_CONSTANT_PROPERTY(TEXTURE1)
	DELETE_CONSTANT_PROPERTY(TEXTURE2)
	DELETE_CONSTANT_PROPERTY(TEXTURE3)
	DELETE_CONSTANT_PROPERTY(TEXTURE4)
	DELETE_CONSTANT_PROPERTY(TEXTURE5)
	DELETE_CONSTANT_PROPERTY(TEXTURE6)
	DELETE_CONSTANT_PROPERTY(TEXTURE7)
	DELETE_CONSTANT_PROPERTY(TEXTURE8)
	DELETE_CONSTANT_PROPERTY(TEXTURE9)
	DELETE_CONSTANT_PROPERTY(TEXTURE10)
	DELETE_CONSTANT_PROPERTY(TEXTURE11)
	DELETE_CONSTANT_PROPERTY(TEXTURE12)
	DELETE_CONSTANT_PROPERTY(TEXTURE13)
	DELETE_CONSTANT_PROPERTY(TEXTURE14)
	DELETE_CONSTANT_PROPERTY(TEXTURE15)
	DELETE_CONSTANT_PROPERTY(TEXTURE16)
	DELETE_CONSTANT_PROPERTY(TEXTURE17)
	DELETE_CONSTANT_PROPERTY(TEXTURE18)
	DELETE_CONSTANT_PROPERTY(TEXTURE19)
	DELETE_CONSTANT_PROPERTY(TEXTURE20)
	DELETE_CONSTANT_PROPERTY(TEXTURE21)
	DELETE_CONSTANT_PROPERTY(TEXTURE22)
	DELETE_CONSTANT_PROPERTY(TEXTURE23)
	DELETE_CONSTANT_PROPERTY(TEXTURE24)
	DELETE_CONSTANT_PROPERTY(TEXTURE25)
	DELETE_CONSTANT_PROPERTY(TEXTURE26)
	DELETE_CONSTANT_PROPERTY(TEXTURE27)
	DELETE_CONSTANT_PROPERTY(TEXTURE28)
	DELETE_CONSTANT_PROPERTY(TEXTURE29)
	DELETE_CONSTANT_PROPERTY(TEXTURE30)
	DELETE_CONSTANT_PROPERTY(TEXTURE31)
	DELETE_CONSTANT_PROPERTY(ACTIVE_TEXTURE)

	/* TextureWrapMode */
	DELETE_CONSTANT_PROPERTY(REPEAT)
	DELETE_CONSTANT_PROPERTY(CLAMP_TO_EDGE)
	DELETE_CONSTANT_PROPERTY(MIRRORED_REPEAT)

	/* Uniform Types */
	DELETE_CONSTANT_PROPERTY(FLOAT_VEC2)
	DELETE_CONSTANT_PROPERTY(FLOAT_VEC3)
	DELETE_CONSTANT_PROPERTY(FLOAT_VEC4)
	DELETE_CONSTANT_PROPERTY(INT_VEC2)
	DELETE_CONSTANT_PROPERTY(INT_VEC3)
	DELETE_CONSTANT_PROPERTY(INT_VEC4)
	DELETE_CONSTANT_PROPERTY(BOOL)
	DELETE_CONSTANT_PROPERTY(BOOL_VEC2)
	DELETE_CONSTANT_PROPERTY(BOOL_VEC3)
	DELETE_CONSTANT_PROPERTY(BOOL_VEC4)
	DELETE_CONSTANT_PROPERTY(FLOAT_MAT2)
	DELETE_CONSTANT_PROPERTY(FLOAT_MAT3)
	DELETE_CONSTANT_PROPERTY(FLOAT_MAT4)
	DELETE_CONSTANT_PROPERTY(SAMPLER_2D)
	DELETE_CONSTANT_PROPERTY(SAMPLER_CUBE)

	/* Vertex Arrays */
	DELETE_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_ENABLED)
	DELETE_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_SIZE)
	DELETE_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_STRIDE)
	DELETE_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_TYPE)
	DELETE_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_NORMALIZED)
	DELETE_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_POINTER)
	DELETE_CONSTANT_PROPERTY(VERTEX_ATTRIB_ARRAY_BUFFER_BINDING)

	/* Read Format */
	DELETE_CONSTANT_PROPERTY(IMPLEMENTATION_COLOR_READ_TYPE)
	DELETE_CONSTANT_PROPERTY(IMPLEMENTATION_COLOR_READ_FORMAT)

	/* Shader Source */
	DELETE_CONSTANT_PROPERTY(COMPILE_STATUS)
	DELETE_CONSTANT_PROPERTY(INFO_LOG_LENGTH)
	DELETE_CONSTANT_PROPERTY(SHADER_SOURCE_LENGTH)
	DELETE_CONSTANT_PROPERTY(SHADER_COMPILER)

	/* Shader Binary */
	DELETE_CONSTANT_PROPERTY(SHADER_BINARY_FORMATS)
	DELETE_CONSTANT_PROPERTY(NUM_SHADER_BINARY_FORMATS)

	/* Shader Precision-Specified Types */
	DELETE_CONSTANT_PROPERTY(LOW_FLOAT)
	DELETE_CONSTANT_PROPERTY(MEDIUM_FLOAT)
	DELETE_CONSTANT_PROPERTY(HIGH_FLOAT)
	DELETE_CONSTANT_PROPERTY(LOW_INT)
	DELETE_CONSTANT_PROPERTY(MEDIUM_INT)
	DELETE_CONSTANT_PROPERTY(HIGH_INT)

	/* Framebuffer Object. */
	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER)

	DELETE_CONSTANT_PROPERTY(RGBA4)
	DELETE_CONSTANT_PROPERTY(RGB5_A1)
	DELETE_CONSTANT_PROPERTY(RGB565)
	DELETE_CONSTANT_PROPERTY(DEPTH_COMPONENT16)
	DELETE_CONSTANT_PROPERTY(STENCIL_INDEX)
	DELETE_CONSTANT_PROPERTY(STENCIL_INDEX8)

	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_WIDTH)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_HEIGHT)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_INTERNAL_FORMAT)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_RED_SIZE)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_GREEN_SIZE)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_BLUE_SIZE)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_ALPHA_SIZE)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_DEPTH_SIZE)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_STENCIL_SIZE)

	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_ATTACHMENT_OBJECT_TYPE)
	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_ATTACHMENT_OBJECT_NAME)
	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_ATTACHMENT_TEXTURE_LEVEL)
	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_ATTACHMENT_TEXTURE_CUBE_MAP_FACE)

	DELETE_CONSTANT_PROPERTY(COLOR_ATTACHMENT0)
	DELETE_CONSTANT_PROPERTY(DEPTH_ATTACHMENT)
	DELETE_CONSTANT_PROPERTY(STENCIL_ATTACHMENT)

	DELETE_CONSTANT_PROPERTY(NONE)

	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_COMPLETE)
	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_INCOMPLETE_ATTACHMENT)
	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT)
	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_INCOMPLETE_DIMENSIONS)
	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_UNSUPPORTED)

	DELETE_CONSTANT_PROPERTY(FRAMEBUFFER_BINDING)
	DELETE_CONSTANT_PROPERTY(RENDERBUFFER_BINDING)
	DELETE_CONSTANT_PROPERTY(MAX_RENDERBUFFER_SIZE)

	DELETE_CONSTANT_PROPERTY(INVALID_FRAMEBUFFER_OPERATION)

	DELETE_CONSTANT_PROPERTY(PSP2_WINDOW_960X544)
	DELETE_CONSTANT_PROPERTY(PSP2_WINDOW_480X272)
	DELETE_CONSTANT_PROPERTY(PSP2_WINDOW_640X368)
	DELETE_CONSTANT_PROPERTY(PSP2_WINDOW_720X408)
	DELETE_CONSTANT_PROPERTY(PSP2_WINDOW_1280X725)
	DELETE_CONSTANT_PROPERTY(PSP2_WINDOW_1920X1088)

	DELETE_CONSTANT_PROPERTY(EGL_DONT_CARE)

	/* Config attributes */
	DELETE_CONSTANT_PROPERTY(EGL_BUFFER_SIZE)
	DELETE_CONSTANT_PROPERTY(EGL_ALPHA_SIZE)
	DELETE_CONSTANT_PROPERTY(EGL_BLUE_SIZE)
	DELETE_CONSTANT_PROPERTY(EGL_GREEN_SIZE)
	DELETE_CONSTANT_PROPERTY(EGL_RED_SIZE)
	DELETE_CONSTANT_PROPERTY(EGL_DEPTH_SIZE)
	DELETE_CONSTANT_PROPERTY(EGL_STENCIL_SIZE)
	DELETE_CONSTANT_PROPERTY(EGL_CONFIG_CAVEAT)
	DELETE_CONSTANT_PROPERTY(EGL_CONFIG_ID)
	DELETE_CONSTANT_PROPERTY(EGL_LEVEL)
	DELETE_CONSTANT_PROPERTY(EGL_MAX_PBUFFER_HEIGHT)
	DELETE_CONSTANT_PROPERTY(EGL_MAX_PBUFFER_PIXELS)
	DELETE_CONSTANT_PROPERTY(EGL_MAX_PBUFFER_WIDTH)
	DELETE_CONSTANT_PROPERTY(EGL_NATIVE_RENDERABLE)
	DELETE_CONSTANT_PROPERTY(EGL_NATIVE_VISUAL_ID)
	DELETE_CONSTANT_PROPERTY(EGL_NATIVE_VISUAL_TYPE)
	DELETE_CONSTANT_PROPERTY(EGL_SAMPLES)
	DELETE_CONSTANT_PROPERTY(EGL_SAMPLE_BUFFERS)
	DELETE_CONSTANT_PROPERTY(EGL_SURFACE_TYPE)
	DELETE_CONSTANT_PROPERTY(EGL_TRANSPARENT_TYPE)
	DELETE_CONSTANT_PROPERTY(EGL_TRANSPARENT_BLUE_VALUE)
	DELETE_CONSTANT_PROPERTY(EGL_TRANSPARENT_GREEN_VALUE)
	DELETE_CONSTANT_PROPERTY(EGL_TRANSPARENT_RED_VALUE)
	DELETE_CONSTANT_PROPERTY(EGL_NONE)	/* Attrib list terminator */
	DELETE_CONSTANT_PROPERTY(EGL_BIND_TO_TEXTURE_RGB)
	DELETE_CONSTANT_PROPERTY(EGL_BIND_TO_TEXTURE_RGBA)
	DELETE_CONSTANT_PROPERTY(EGL_MIN_SWAP_INTERVAL)
	DELETE_CONSTANT_PROPERTY(EGL_MAX_SWAP_INTERVAL)
	DELETE_CONSTANT_PROPERTY(EGL_LUMINANCE_SIZE)
	DELETE_CONSTANT_PROPERTY(EGL_ALPHA_MASK_SIZE)
	DELETE_CONSTANT_PROPERTY(EGL_COLOR_BUFFER_TYPE)
	DELETE_CONSTANT_PROPERTY(EGL_RENDERABLE_TYPE)
	DELETE_CONSTANT_PROPERTY(EGL_MATCH_NATIVE_PIXMAP)	/* Pseudo-attribute (not queryable) */
	DELETE_CONSTANT_PROPERTY(EGL_CONFORMANT)

	/* Reserved 0x3041-0x304F for additional config attributes */

	/* Config attribute values */
	DELETE_CONSTANT_PROPERTY(EGL_SLOW_CONFIG)	/* EGL_CONFIG_CAVEAT value */
	DELETE_CONSTANT_PROPERTY(EGL_NON_CONFORMANT_CONFIG)	/* EGL_CONFIG_CAVEAT value */
	DELETE_CONSTANT_PROPERTY(EGL_TRANSPARENT_RGB)	/* EGL_TRANSPARENT_TYPE value */
	DELETE_CONSTANT_PROPERTY(EGL_RGB_BUFFER)	/* EGL_COLOR_BUFFER_TYPE value */
	DELETE_CONSTANT_PROPERTY(EGL_LUMINANCE_BUFFER)	/* EGL_COLOR_BUFFER_TYPE value */

	/* More config attribute values, for EGL_TEXTURE_FORMAT */
	DELETE_CONSTANT_PROPERTY(EGL_NO_TEXTURE)
	DELETE_CONSTANT_PROPERTY(EGL_TEXTURE_RGB)
	DELETE_CONSTANT_PROPERTY(EGL_TEXTURE_RGBA)
	DELETE_CONSTANT_PROPERTY(EGL_TEXTURE_2D)

	/* Config attribute mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_PBUFFER_BIT)	/* EGL_SURFACE_TYPE mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_PIXMAP_BIT)	/* EGL_SURFACE_TYPE mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_WINDOW_BIT)	/* EGL_SURFACE_TYPE mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_VG_COLORSPACE_LINEAR_BIT)	/* EGL_SURFACE_TYPE mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_VG_ALPHA_FORMAT_PRE_BIT)	/* EGL_SURFACE_TYPE mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_MULTISAMPLE_RESOLVE_BOX_BIT)	/* EGL_SURFACE_TYPE mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_SWAP_BEHAVIOR_PRESERVED_BIT)	/* EGL_SURFACE_TYPE mask bits */

	DELETE_CONSTANT_PROPERTY(EGL_OPENGL_ES_BIT)	/* EGL_RENDERABLE_TYPE mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_OPENVG_BIT)	/* EGL_RENDERABLE_TYPE mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_OPENGL_ES2_BIT)	/* EGL_RENDERABLE_TYPE mask bits */
	DELETE_CONSTANT_PROPERTY(EGL_OPENGL_BIT)	/* EGL_RENDERABLE_TYPE mask bits */
}